#!/bin/sh

# License {{{1

# Copyright (c) 2009, Aldis Berjoza <killasmurf86@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following disclaimer
#   in the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the  nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# 1}}}

PLAYD_VERSION="1.3.8"
PLAYD_NAME=`basename $0`

# HOME variable must be defined
if [ "$HOME" = '' ]; then
	echo 'ERR: You are homeless. $HOME not defined' >&2
	exit 1
fi

PLAYD_HOME=${PLAYD_HOME="$HOME/.mplayer"}
PLAYD_PIPE="$PLAYD_HOME/playd.fifo"
PLAYD_PLAYLIST="$PLAYD_HOME/playd_playlist.plst"
PLAYD_LOCK="$PLAYD_HOME/playd.lock"

# to customize mplayers cmdl set PLAYD_MPLAYER_USER_OPTIONS env variable
MPLAYER_CMD_GENERIC="${PLAYD_MPLAYER_USER_OPTIONS} -idle -input file=$PLAYD_PIPE"
#MPLAYER_CMD_GENERIC="${PLAYD_MPLAYER_USER_OPTIONS} -idle -quiet -input file=$PLAYD_PIPE"
MPLAYER_CMD="mplayer $MPLAYER_CMD_GENERIC"
MPLAYER_SND_ONLY_CMD="mplayer -vo null $MPLAYER_CMD_GENERIC"


# playd_help() {{{1
playd_help() {
# print help
	cat << EOF
$PLAYD_NAME v$PLAYD_VERSION
by Aldis Berjoza <aldis@bsdroot.lv>
http://github.com/killasmurf86/playd

$PLAYD_NAME is script to control mplayer in slave mode

Special thanks to DutchDaemon and blah from forums.freebsd.org for few lines of sh
EOF
}
# 1}}}

# playd_put() {{{1
playd_put() {
# put argv into pipe
	playd_check
	if [ $? -gt 0 ]; then
		echo "$*" >> "$PLAYD_PIPE"
		return
	fi
	echo "ERR: playd doesn't seam to be running" >&2
	exit 1
}
# 1}}}

# playd_check() {{{1
playd_check() {
# check if playd daemon is running and return pid
# returns 0 if daemon ain't running
	if [ -f "$PLAYD_LOCK" ]; then
		local pidStored=`cat $PLAYD_LOCK`
		for pid in `pgrep mplayer`; do
			if [ "$pid" = "$pidStored" ]; then return "$pidStored"; fi
		done
		playd_clean
	fi
	return 0
}
# 1}}}

# playd_clean() {{{1
playd_clean() {
# clean files after playd
	rm -f "$PLAYD_PLAYLIST.tmp"
	rm -f "$PLAYD_PIPE"
	rm -f "$PLAYD_LOCK"
}
# 1}}}

# playd_start() {{{1
playd_start() {
# start daemon
# possible artguments:
# console novid (order doesn't matter)
	playd_check
	if [ $? -eq 0 ]; then
		if [ ! -p "$PLAYD_PIPE" ]; then
			mkfifo "$PLAYD_PIPE"
			if [ $? -gt 0 ]; then
				echo "ERR: Can't create $PLAYD_HOME" >&2
				exit 1
			fi
		else
			echo "ERR: There may be another instance of playd running. Run playd --restart" >&2
			exit 1
		fi
		cd /
		if [ $NOVID -eq 0 ]; then
			local mplayer_run_cmd="$MPLAYER_CMD"
		else
			local mplayer_run_cmd="$MPLAYER_SND_ONLY_CMD"
		fi

		if [ $CONSOLE -eq 0 ]; then
			exec ${mplayer_run_cmd} > /dev/null 2> /dev/null &
			echo "$!" > "$PLAYD_LOCK"
		else
			local dbg_pipe="$PLAYD_HOME/playd_debug.pipe"
			mkfifo "$dbg_pipe"
			exec ${mplayer_run_cmd} > "$dbg_pipe" 2> "$dbg_pipe" &
			echo "$!" > "$PLAYD_LOCK"
			cat "$dbg_pipe"
			playd_clean
			exit
		fi
		cd -
	fi
}
# 1}}}

# playd_stop() {{{1
playd_stop() {
# stop playd daemon
	playd_check
	if [ $? -gt 0 ]; then
		playd_put 'quit'
		if [ $? -gt 0 ]; then
			sleep 1 # give mplayer 1s to quit
			playd_check
			local kill_pid=$?
			if [ $kill_pid -gt 0 ]; then
				kill $kill_pid
				sleep 1
				playd_check
				kill_pid=$?
				if [ $kill_pid -gt 0 ]; then
					kill -s KILL $kill_pid
					sleep 1
					playd_check
					if [ $? -gt 0 ]; then
						echo "ERR: can't kill mplayer slave on pid $kill_pid" >&2
						exit 1
					fi
				fi
			fi
		fi
	fi
	playd_clean
}
# 1}}}

# playd_mk_playlist() {{{1
playd_mk_playlist() {
# make playlist form directory
# arguments:
# $1 - dir to make list
	# $1 and $fileName must be double quoted to avoid
	#   problems with filenames that contain speccial characters
	ls "$1" | while read fileName; do
		if [ -f "$1/$fileName" ]; then
			local fname="$1/$fileName"
			case `echo ${fname##*.} | tr 'QWERTYUIOPASDFGHJKLZXCVBNM' 'qwertyuiopasdfghjklzxcvbnm'` in
				mp3|flac|ogg|oga|aac|wav|wma|aiff|aifc|m4a|m4b|m4p|m4r|mpga|mka|avi|mkv|vob|mp4|3gp|wmv|mov|divx|dvx|ogm|ogv|ape )
					echo "$fname" >> "$PLAYD_PLAYLIST.tmp"
				;;

				*)
					case `file -ib "$fname" | tr ' ' '_'` in
						'audio'* | 'video'* )
							echo "$fname" >> "$PLAYD_PLAYLIST.tmp"
						;;
					esac
			esac
		elif [ -d "$1/$fileName" ]; then
			playd_mk_playlist "$1/$fileName"
		else
			echo 'ERR: wtf?' >&2
			echo "file: $1/$fileName" >&2
			exit 1
		fi
	done
}
# 1}}}

# playd_match() {{{1
playd_match() {
# playd match takes at least 4 arguments.
# argument count must be even
# arg 1 is key we need to find
# arg 2 is what we return if we don't find key
#---
# next argument is what we return when we find key
# next argument is quoted list of possible keys
#---
# etc
#=======
# check it's usage in source (especially where we want to set volume)
	if [ $# -ge 4 ]; then
		if [ `expr $# % 2` -eq 0 ]; then
			local mkey="$1" # search for key
			local errVal="$2" # return if not found
			while [ $# -gt 0 ]; do
				shift 2
				for ckey in $2; do
					if [ "$mkey" = "$ckey" ]; then
						echo "$1"
						return
					fi
				done
			done
			echo "$errVal"
		else
			echo 'ERR: playd_match takes even numver of arguments' >&2
			exit 255
		fi
	else
		echo 'ERR: playd_match takes at least 4 arguments' >&2
		exit 255
	fi
}
# 1}}}

# playd_fullpath() {{{1
playd_fullpath() {
# echo full path of file/dir
# should be use as function
	case "$1" in
		/* ) echo "$1";;
		* ) echo "`pwd`/$1";;
	esac
}
# 1}}}

# playd_playlist_add() {{{1
playd_playlist_add() {
#add entry to playlist
# arg1 = playlist item
	playd_start
	playd_put "loadfile $1 $playd_append"
	if [ $playd_append -eq 1 ]; then
		echo "$*" >> "$PLAYD_PLAYLIST"
	else
		echo "$*" > "$PLAYD_PLAYLIST"
	fi
	playd_append=1
}
# 1}}}


# checking for mplayer
if [ ! "$(which mplayer)" ]; then
	echo 'ERR: mplayer not found' >&2
	exit 1
fi

if [ ! -d "$PLAYD_HOME" ]; then
	mkdir "$PLAYD_HOME"
	if [ $? -gt 0 ]; then
		echo "ERR: Can't create $PLAYD_HOME" >&2
		exit 1
	fi
fi


if [ $# -eq 0 ]; then
	playd_help
fi

CONSOLE=0
NOVID=0

playd_append=$(playd_match "$1" 0 1 'append --append -a')
if [ $playd_append -eq 1 ]; then shift; fi


# check command line arguments
while [ $# -gt 0 ]; do
	case "$1" in
		'append' | '--append' | '-a' )
			echo "WARN: $1 should be 1st argument. Ignoring" >&2
		;;

		'help' | '--help' | '-h')
			playd_help
		;;

		'stop' | '--stop' | '-q')
			playd_stop
		;;

		'start' | '--start' \
		| 'restart' | '--restart' | '-R' )
			if [ $(playd_match "$1" '0' '1' 'restart --restart -R') ]; then playd_stop; fi
			NOVID=$(playd_match "$2" '0' '1' 'novid --novid')
			CONSOLE=$(playd_match "$2" '0' '1' 'console --console')
			shift $(expr $NOVID + $CONSOLE)
			playd_start ${match1} ${match2}
		;;

		'list' | '--list' | '-l' )
			cat "$PLAYD_PLAYLIST" | awk '{print  "\""$0"\""}' | xargs basename | awk '{print NR,")",$0}'
		;;

		'--longlist' | 'longlist' | 'llist' | '--llist' | '-L' )
			cat "$PLAYD_PLAYLIST" | awk '{print NR,")",$0}'
		;;

# seams buggy
# I think there's mplayer bug
# after using playd next or playd seek, playd play dosn't work well (if at all)
		'play' | '--play' | '-p' )
			if [ -n $2 -a $2 -gt 0 ]; then
				playd_start
				while [ "$2" != '' ]; do
					if [ -n $2 -a $2 -gt 0 ]; then
						playd_put `cat $PLAYD_PLAYLIST | awk '{ if (NR == '$2' ) item = $0 } END { print "loadfile " "\""item"\" '$playd_append'"}'`
						shift
					else
						break
					fi
				done
			else
				echo "WARN: $1 needs numberic agument. Ignoring" >&2
			fi
		;;

		'playlist' | '--playlist' | '-P' )
			if [ -f "$2" ]; then
				playd_start
				if [ `dirname $(file -ib $2 | tr ' ' '_')` = 'text' ]; then
					playd_put "loadlist '$2' $playd_append"
					shift
				else
					playd_put "loadlist '$PLAYD_PLAYLIST' $playd_append"
				fi
				playd_append=1
			elif [ -d "$2" ]; then
				echo "WARN: not playlist '$2'. Ignoring" >&2
			else
				playd_start
				playd_put "loadlist '$PLAYD_PLAYLIST' $playd_append"
				playd_append=1
			fi
		;;

		'rmlist' | '--rmlist' )
			rm -f "$PLAYD_PLAYLIST"
		;;


		'seek' | '--seek' | '-s' )
			if [ -n $2 ]; then
				match=$(playd_match "$3" 0 2 'abs --absolute absolute' 1 '% --percent percent')
				playd_put "seek $2 ${match}"
				if [ $match -ne 0 ]; then shift; fi
				shift
			else
				echo "WARN: $1 needs numberic atgument. Ignoring" >&2
			fi
		;;

		'next' | '--next' | '-n' )
			playd_put "seek 100 1"
		;;

		'status' | '--status' )
			playd_check
			pid=$?
			if [ $pid -gt 0 ]; then
				echo "playd is running. PID: $pid"
			else
				echo 'playd is not running'
			fi
		;;

		'cd' | 'cdda' | '--cd' | '-c' \
		| 'dvd' | '--dvd' | '-d' )
			media=$(playd_match $1 '0' 'cdda://' 'cd cdda --cd -c' 'dvd://' 'dvd --dvd -d')
			if [ -n $2 -a $2 -gt 0 ]; then
				while [ "$2" != '' ]; do
					if [ -n $2 -a $2 -gt 0 ]; then
						playd_playlist_add "'${media}$2'"
						shift
					else
						break
					fi
				done
			else
				playd_playlist_add "'${media}'"
			fi
		;;

		'cmd' | '--cmd' )
			if [ "$2" != '' ]; then
				playd_put "$2"
				shift
			else
				echo "WARN: $1 needs argument to pass to mplayer. Ignoring" >&2
			fi
		;;

		'nocheck' | '--nocheck' )
			if [ -f "$2" ]; then
				playd_playlist_add "'$(playd_fullpath $2)'"
				shift
			else
				echo "WARN: '$2' directory. Skipping" >&2
			fi
		;;

		'--subtitles' | 'subtitles' | '--subs' | 'subs' | '-S' )
			if [ -f "$2" ]; then
				playd_put "sub_load '$2'"
				shift
			else
				echo "WARN: '$2' isnt's subtitle file. Skipping" >&2
			fi
		;;

		'brightness' | '--brightness' \
		| 'contrast' | '--contrast' \
		| 'gamma' | '--gamma' \
		| 'hue' | '--hue' \
		| 'saturation' | '--saturation' \
		| 'volume' | '--volume' | 'vol' | '-V' | '--vol' \
		| '--audio-delay' | 'audio-delay' \
		)
			if [ -n $2 ]; then
				match=$(playd_match "$3" 0 1 'abs --absolute absolute')
				playd_put "$(playd_match \
					"$1" \
					"$1" \
					'volume' 'vol -V --vol volume --volume' \
					'audio_delay' '--audio-delay audio-delay' \
					| tr -d '-') $2 ${match}"
				shift $(expr 1 + $match)
			else
				echo "WARN: $1 needs at least numberic argument. Ignoring" >&2
			fi
		;;

		'mute' | '--mute' | '-m' \
		| 'pause' | '--pause' | '-z' \
		| '--switch-audio' | 'switch-audio' | '--sw-audio' | 'sw-audio' \
		| '--switch-subtitles' | 'switch-subtitles' | '--sw-subs' | 'sw-subs' \
		)
			playd_put "$(playd_match \
				"$1" \
				"$1" \
				'mute' 'mute --mute -m' \
				'pause' 'pause --pause -z' \
				'switch_audio' '--switch-audio switch-track --sw-audio sw-audio' \
				'sub_select' '--switch-subtitles switch-subtitles --sw-subs sw-subs' \
				| tr -d '-' )"
		;;

		*'://'* )
			playd_playlist_add "$1"
		;;

		'file' | '--file' | '-f' | * )
			if [ $(playd_match "$1" 0 1 'file --file -f') -eq 1 ]; then shift; fi


			if [ -f "$1" -o -d "$1" ]; then
				isPlayable=0
				playd_start
				fileName=$(playd_fullpath $1)

			case `echo ${1##*.} | tr 'QWERTYUIOPASDFGHJKLZXCVBNM' 'qwertyuiopasdfghjklzxcvbnm'` in \
					mp3|flac|ogg|oga|aac|wav|wma|aiff|aifc|m4a|m4b|m4p|m4r|mpga|mka|avi|mkv|vob|mp4|3gp|wmv|mov|divx|dvx|ogm|ogv )
						isPlayable=1
					;;

					*)
						case `file -ib "$fileName" | tr ' ' '_'` in
							'audio'* | 'video'* )
								isPlayable=1
							;;

							'application/x-directory'* )
								rm -f "$PLAYD_PLAYLIST.tmp"
								playd_mk_playlist "$fileName"
								playd_put "loadlist '$PLAYD_PLAYLIST.tmp' $playd_append"
								if [ $playd_append -eq 1 ]; then
									cat "$PLAYD_PLAYLIST.tmp" >> "$PLAYD_PLAYLIST"
								else
									cat "$PLAYD_PLAYLIST.tmp" > "$PLAYD_PLAYLIST"
									playd_append=1
								fi
							;;

							*)
								echo "WARN: '$fileName' doesn't seam to be vailid file for playback" >&2
								echo "WARN: to override use:" >&2
								echo "  playd --nocheck $fileName" >&2
							;;
						esac
					;;
				esac

				if [ $isPlayable -eq 1 ]; then
					playd_playlist_add "'$fileName'"
				fi

			else
				echo "WARN: not a file/directory: '$1'" >&2
			fi
		;;

	esac

	shift
done

exit 0

# vim:tabstop=4:shiftwidth=4:
